<?php

use WHMCS\Domain\TopLevel\ImportItem;
use WHMCS\Results\ResultsList;

class bnamed_conn {

    var $BaseUrl = "https://api.bNamed.net/API?";
    var $ConnUrl = '';
    var $RequestResponse = '';
    var $ParsedResponse = '';
    var $Error = '';

    function CreateConnUrl($uid, $key) {
        $this->ConnUrl = $this->BaseUrl . "uid=" . $uid . "&key=" . $key . "&WHMCSversion=whmcs20230728";
    }

    function AddParam($name, $value) {
        $this->ConnUrl .= "&" . $name . "=" . urlencode($value);
    }

    function SendRequest() {
        $this->Error = "Incomplete request";
        try {
            $this->RequestResponse = file_get_contents($this->ConnUrl);
            $this->ParsedResponse = simplexml_load_string($this->RequestResponse);

            if (empty($this->ParsedResponse)) {
                $this->Error = "Invalid response received";
            } elseif ($this->ParsedResponse->ErrorCode > 0) {
                $this->Error = $this->ParsedResponse->ErrorText . " ( ErrorCode = " . $this->ParsedResponse->ErrorCode . " )";
            } elseif ($this->ParsedResponse->ErrorCode == 0) {
                $this->Error = "";
            } else {
                $this->Error = "Unknown error";
            }
        } catch (Error $e) {
            echo "gevangen";
            $this->Error = $e->getMessage();
        } catch (Exception $e) {
            echo "gevangen";
            $this->Error = $e->getMessage();
        }

        if ($this->Error == "") {
            return true;
        } else {
            return false;
        }
    }

    function ParseFloat($inputString) {
        return (float) str_replace(",", ".", (string) $inputString);
    }

    function CreateDNSList($params) {
        $dnslist = $params["ns1"];
        if ($params["ns2"] != '')
            $dnslist .= "," . $params["ns2"];
        if ($params["ns3"] != '')
            $dnslist .= "," . $params["ns3"];
        if ($params["ns4"] != '')
            $dnslist .= "," . $params["ns4"];
        if ($params["ns5"] != '')
            $dnslist .= "," . $params["ns5"];

        return $dnslist;
    }

}

function bnamed_getConfigArray() {
    $configarray = array(
        "UID" => array("Type" => "text", "Size" => "20", "Description" => "Enter your bNamed UID here",),
        "Key" => array("Type" => "password", "Size" => "20", "Description" => "Enter your bNamed Key here",),
        "auth_id" => array("FriendlyName" => "ClouDNS UID", "Type" => "text", "Size" => "20", "Description" => "Enter your ClouDNS UID here",),
        "auth_password" => array("FriendlyName" => "ClouDNS Password", "Type" => "text", "Size" => "20", "Description" => "Enter your ClouDNS password here",),
        "Nameserver1" => array("Type" => "text", "Size" => "20", "Description" => "(default nameserver 1)",),
        "Nameserver2" => array("Type" => "text", "Size" => "20", "Description" => "(default nameserver 2)",),
        "Nameserver3" => array("Type" => "text", "Size" => "20", "Description" => "(default nameserver 3)",),
        "Nameserver4" => array("Type" => "text", "Size" => "20", "Description" => "(default nameserver 4)",),
    );
    return $configarray;
}

function bnamed_GetNameservers($params) {

    $username = $params["UID"];
    $password = $params["Key"];
    $tld = $params["tld"];
    $sld = $params["sld"];

    $bNamed = new bnamed_conn();

    $bNamed->CreateConnUrl($username, $password);
    $bNamed->AddParam("command", "GetDomain");
    $bNamed->AddParam("sld", $sld);
    $bNamed->AddParam("tld", $tld);
    $values = array();

    if ($bNamed->SendRequest()) {
        $nameservers = explode(",", $bNamed->ParsedResponse->Result->DNSList);

        $values["ns1"] = $nameservers[0];
        $values["ns2"] = $nameservers[1];
        $values["ns3"] = $nameservers[2];
        $values["ns4"] = $nameservers[3];
        $values["ns5"] = $nameservers[4];
    } else {
        $values["error"] = $bNamed->Error;
    }

    return $values;
}

function bnamed_SaveNameservers($params) {
    $username = $params["UID"];
    $password = $params["Key"];
    $tld = $params["tld"];
    $sld = $params["sld"];
    $values = array();

    if(strrpos($params["ns1"], ".obambu.com") || !strrpos($params["ns1"], ".obambu.")) {

        $bNamed = new bnamed_conn();

        $bNamed->CreateConnUrl($username, $password);
        $bNamed->AddParam("command", "Update");
        $bNamed->AddParam("sld", $sld);
        $bNamed->AddParam("tld", $tld);
        $bNamed->AddParam("dnstype", "E");
        $bNamed->AddParam("dnslist", $bNamed->CreateDNSList($params));

        if (!$bNamed->SendRequest()) {
            $$error = $bNamed->Error;
        }
    } else {
        $error = "Using our nameservers, nothing changed";
    }

    $values["error"] = $error;
    return $values;
}

function bnamed_GetRegistrarLock($params) {
    $username = $params["UID"];
    $password = $params["Key"];
    $tld = $params["tld"];
    $sld = $params["sld"];

    $bNamed = new bNamed_conn();

    $bNamed->CreateConnUrl($username, $password);
    $bNamed->AddParam("command", "GetDomain");
    $bNamed->AddParam("sld", $sld);
    $bNamed->AddParam("tld", $tld);

    if ($bNamed->SendRequest()) {
        if (trim($bNamed->ParsedResponse->Result->TransferLock) == "true") {
            $lockstatus = 'locked';
        } else {
            $lockstatus = 'unlocked';
        }
        return $lockstatus;
    } else {
        return array(
            'error' => $bNamed->Error
        );
    }
}

function bnamed_SaveRegistrarLock($params) {
    $username = $params["UID"];
    $password = $params["Key"];
    $tld = $params["tld"];
    $sld = $params["sld"];

    $bNamed = new bNamed_conn();

    $bNamed->CreateConnUrl($username, $password);
    $bNamed->AddParam("command", "lock");
    $bNamed->AddParam("sld", $sld);
    $bNamed->AddParam("tld", $tld);
    $bNamed->AddParam("lock", ($params["lockenabled"] == "locked" ? "1" : "0"));
    $bNamed->AddParam("noEmail", "1");

    if ($bNamed->SendRequest()) {
        $values = array(
            'success' => 'success'
        );
    } else {
        $values = array(
            'error' => $bNamed->Error
        );
    }

    return $values;
}

function bnamed_RegisterDomain($params) {
    $username = $params["UID"];
    $password = $params["Key"];
    $tld = $params["tld"];
    $sld = $params["sld"];
    $regperiod = $params["regperiod"];
    $values = array();

    $bNamed = new bnamed_conn();

    $bNamed->CreateConnUrl($username, $password);
    $bNamed->AddParam("command", "purchase");
    $bNamed->AddParam("sld", $sld);
    $bNamed->AddParam("tld", $tld);
    $bNamed->AddParam("PurchaseType", "N");

    if ($params['idprotection']) {
        $bNamed->AddParam("whoisprivacy", "1");
    } else {
        $bNamed->AddParam("whoisprivacy", "0");
    }

    $address_arr = explode(" ", $params["address1"]);
    $nr = $address_arr[count($address_arr) - 1];
    $street = '';
    for ($i = 0; $i < count($address_arr) - 1; $i++) {
        $street .= " " . $address_arr[$i];
    }

    $bNamed->AddParam("LicenseeNr", "-2");
    $bNamed->AddParam("LicenseeFirstName", $params["firstname"]);
    $bNamed->AddParam("LicenseeLastName", $params["lastname"]);
    $bNamed->AddParam("LicenseeStreet", $street);
    $bNamed->AddParam("LicenseeStreetNr", $nr);
    $bNamed->AddParam("LicenseePostCode", $params["postcode"]);
    $bNamed->AddParam("LicenseeResidence", $params["city"]);
    $bNamed->AddParam("LicenseeCountry", $params["country"]);
    $bNamed->AddParam("LicenseePhone", $params["phonenumber"]);
    $bNamed->AddParam("LicenseeEMail", $params["email"]);
    $bNamed->AddParam("LicenseeCompany", $params["companyname"]);

    $address_arr = explode(" ", $params["adminaddress1"]);
    $nr = $address_arr[count($address_arr) - 1];
    $street = '';
    for ($i = 0; $i < count($address_arr) - 1; $i++) {
        $street .= " " . $address_arr[$i];
    }

    $bNamed->AddParam("OnSiteNr", "-2");
    $bNamed->AddParam("OnSiteFirstName", $params["adminfirstname"]);
    $bNamed->AddParam("OnSiteLastName", $params["adminlastname"]);
    $bNamed->AddParam("OnSiteStreet", $street);
    $bNamed->AddParam("OnSiteStreetNr", $nr);
    $bNamed->AddParam("OnSitePostCode", $params["adminpostcode"]);
    $bNamed->AddParam("OnSiteResidence", $params["admincity"]);
    $bNamed->AddParam("OnSiteCountry", $params["admincountry"]);
    $bNamed->AddParam("OnSitePhone", $params["adminphonenumber"]);
    $bNamed->AddParam("OnSiteEMail", $params["adminemail"]);
    $bNamed->AddParam("OnSiteCompany", $params["admincompanyname"]);

    if ($tld == "it") {
        $bNamed->AddParam("BTWNR", $params['additionalfields']['VATNR']);
        $bNamed->AddParam("IDNumber", $params['additionalfields']['IDNumber']);
        $bNamed->AddParam("StateProvince", $params['additionalfields']['StateProvince']);
        $bNamed->AddParam("entityType", $params['additionalfields']['entityType']);
    }
    if ($tld == "si") {
        if (strlen($params['additionalfields']['VATNR']) > 0) {
            $bNamed->AddParam("BTWNR", $params['additionalfields']['VATNR']);
        } else {
            $bNamed->AddParam("BTWNR", $params["customfields1"]);
        }
    }

    $bNamed->AddParam("NumYears", $regperiod);
    $bNamed->AddParam("DNSType", "E");
    $bNamed->AddParam("ExtendType", "0");
    $bNamed->AddParam("DNSList", $bNamed->CreateDNSList($params));

    $bNamed->ConnUrl = str_replace(" ", "%20", $bNamed->ConnUrl);

    if (!$bNamed->SendRequest()) {
        $values["error"] = $bNamed->Error;
    }

    return $values;
}

function bnamed_TransferDomain($params) {
    $username = $params["UID"];
    $password = $params["Key"];
    $tld = $params["tld"];
    $sld = $params["sld"];
    $regperiod = $params["regperiod"];
    $values = array();

    $RegistrantFirstName = $params["firstname"];
    $RegistrantLastName = $params["lastname"];
    $RegistrantCompanyName = $params["companyname"];
    $RegistrantAddress1 = $params["address1"];
    $RegistrantCity = $params["city"];
    $RegistrantPostalCode = $params["postcode"];
    $RegistrantCountry = $params["country"];
    $RegistrantEmailAddress = $params["email"];
    $RegistrantPhone = $params["phonenumber"];

    $bNamed = new bnamed_conn();

    $bNamed->CreateConnUrl($username, $password);
    $bNamed->AddParam("command", "purchase");
    $bNamed->AddParam("sld", $sld);
    $bNamed->AddParam("tld", $tld);
    $bNamed->AddParam("PurchaseType", "B");

    if ($tld == "it") {
        $bNamed->AddParam("BTWNR", $params['additionalfields']['VATNR']);
        $bNamed->AddParam("IDNumber", $params['additionalfields']['IDNumber']);
        $bNamed->AddParam("StateProvince", $params['additionalfields']['StateProvince']);
        $bNamed->AddParam("entityType", $params['additionalfields']['entityType']);
    }
    if ($tld == "si") {
        if (strlen($params['additionalfields']['VATNR']) > 0) {
            $bNamed->AddParam("BTWNR", $params['additionalfields']['VATNR']);
        } else {
            $bNamed->AddParam("BTWNR", $params["customfields1"]);
        }
    }

    if ($params['protectenable']) {
        $bNamed->AddParam("whoisprivacy", "1");
    } else {
        $bNamed->AddParam("whoisprivacy", "0");
    }

    if ($params["customfields2"] && $params["customfields2"] != "") {
        $bNamed->AddParam("LicenseeNr", $params["customfields2"]);
    } else {
        $address_arr = explode(" ", $RegistrantAddress1);
        $nr = $address_arr[count($address_arr) - 1];
        $street = '';
        for ($i = 0; $i < count($address_arr) - 1; $i++) {
            $street .= $address_arr[$i];
        }
        $bNamed->AddParam("LicenseeNr", "-2");
        $bNamed->AddParam("LicenseeFirstName", $RegistrantFirstName);
        $bNamed->AddParam("LicenseeLastName", $RegistrantLastName);
        $bNamed->AddParam("LicenseeStreet", $street);
        $bNamed->AddParam("LicenseeStreetNr", $nr);
        $bNamed->AddParam("LicenseeStreet", $RegistrantAddress1);
        $bNamed->AddParam("LicenseePostCode", $RegistrantPostalCode);
        $bNamed->AddParam("LicenseeResidence", $RegistrantCity);
        $bNamed->AddParam("LicenseeCountry", $RegistrantCountry);
        $bNamed->AddParam("LicenseePhone", $RegistrantPhone);
        if ($RegistrantCountry == "BE")
            $cc = "32";
        elseif ($RegistrantCountry == "NL")
            $cc = "31";
        else
            $cc = "00";
        $bNamed->AddParam("LicenseeCountryCode", $cc);
        $bNamed->AddParam("LicenseeEMail", $RegistrantEmailAddress);
        $bNamed->AddParam("LicenseeCompany", $RegistrantCompanyName);
    }
    $bNamed->AddParam("OnSiteNr", $params["UID"]);
    $bNamed->AddParam("NumYears", $regperiod);
    $bNamed->AddParam("ExtendType", "0");
    $bNamed->AddParam("DNSType", "E");
    if ($params["transfersecret"] != '')
        $bNamed->AddParam("AuthKey", $params["transfersecret"]);
    $bNamed->AddParam("DNSList", $bNamed->CreateDNSList($params));

    if (!$bNamed->SendRequest()) {
        $values["error"] = $bNamed->Error;
    }

    return $values;
}

function bnamed_RenewDomain($params) {
    $username = $params["UID"];
    $password = $params["Key"];
    $tld = $params["tld"];
    $sld = $params["sld"];
    $error = '';

    try {
        $query = @mysql_query("SELECT expirydate FROM tbldomains WHERE id = '" . $params["domainid"] . "'");
        if (@mysql_num_rows($query) != 0) {
            $result = @mysql_fetch_array($query);
            $ultimateDate = $oneYearOn = date('Y-m-d', strtotime($result['expirydate'] . "+" . $params["regperiod"] . " year"));
            $params["debugExpirydate"] = $result['expirydate'];
        }
    } catch (Exception $e) {
        $error = $e->getMessage();
    } catch (Error $e) {
        $error = $e->getMessage();
    }

    if ($error == '') {
        $bNamed = new bnamed_conn();

        $bNamed->CreateConnUrl($username, $password);
        $bNamed->AddParam("command", "extend");
        $bNamed->AddParam("sld", $sld);
        $bNamed->AddParam("tld", $tld);
        $bNamed->AddParam("newExpiration", $ultimateDate);

        //debug
        $bNamed->AddParam("debug", base64_encode(print_r($params, true)));

        if (!$bNamed->SendRequest()) {
            $error = $bNamed->Error;
        }
    }

    $values["error"] = $error;
    return $values;
}

function bnamed_GetContactDetails($params) {
    $username = $params["UID"];
    $password = $params["Key"];
    $tld = $params["tld"];
    $sld = $params["sld"];
    $values = array();

    $bNamed = new bnamed_conn();

    $bNamed->CreateConnUrl($username, $password);
    $bNamed->AddParam("command", "GetDomain");
    $bNamed->AddParam("sld", $sld);
    $bNamed->AddParam("tld", $tld);

    if ($bNamed->SendRequest()) {
        $values["Registrant"]["First Name"] = $bNamed->ParsedResponse->Result->Licensee->FirstName . "";
        $values["Registrant"]["Last Name"] = $bNamed->ParsedResponse->Result->Licensee->LastName . "";
        $values["Registrant"]["Company"] = $bNamed->ParsedResponse->Result->Licensee->Company . "";
        $values["Registrant"]["Address1"] = $bNamed->ParsedResponse->Result->Licensee->StreetNr . " " . $bNamed->ParsedResponse->Result->Licensee->Street;
        $values["Registrant"]["Postcode"] = $bNamed->ParsedResponse->Result->Licensee->Postcode . "";
        $values["Registrant"]["City"] = $bNamed->ParsedResponse->Result->Licensee->Residence . "";
        $values["Registrant"]["Country"] = $bNamed->ParsedResponse->Result->Licensee->Country . "";
        $values["Registrant"]["Phone"] = $bNamed->ParsedResponse->Result->Licensee->Phone . "";
        $values["Registrant"]["E-mail"] = $bNamed->ParsedResponse->Result->Licensee->EMail . "";
        $values["Registrant"]["Fax"] = $bNamed->ParsedResponse->Result->Licensee->Fax . "";

        $values["Tech"]["First Name"] = $bNamed->ParsedResponse->Result->OnSite->FirstName . "";
        $values["Tech"]["Last Name"] = $bNamed->ParsedResponse->Result->OnSite->LastName . "";
        $values["Tech"]["Company"] = $bNamed->ParsedResponse->Result->OnSite->Company . "";
        $values["Tech"]["Address1"] = $bNamed->ParsedResponse->Result->OnSite->StreetNr . " " . $bNamed->ParsedResponse->Result->OnSite->Street;
        $values["Tech"]["Postcode"] = $bNamed->ParsedResponse->Result->OnSite->Postcode . "";
        $values["Tech"]["City"] = $bNamed->ParsedResponse->Result->OnSite->Residence . "";
        $values["Tech"]["Country"] = $bNamed->ParsedResponse->Result->OnSite->Country . "";
        $values["Tech"]["Phone"] = $bNamed->ParsedResponse->Result->OnSite->Phone . "";
        $values["Tech"]["E-mail"] = $bNamed->ParsedResponse->Result->OnSite->EMail . "";
        $values["Tech"]["Fax"] = $bNamed->ParsedResponse->Result->OnSite->Fax . "";
    } else {
        $values["error"] = $bNamed->Error;
    }

    return $values;
}

function bnamed_SaveContactDetails($params) {
    $username = $params["UID"];
    $password = $params["Key"];
    $tld = $params["tld"];
    $sld = $params["sld"];
    $values = array();

    $bNamed = new bnamed_conn();

    $bNamed->CreateConnUrl($username, $password);
    $bNamed->AddParam("command", "update");
    $bNamed->AddParam("sld", $sld);
    $bNamed->AddParam("tld", $tld);

    $bNamed->AddParam("LicenseeNr", "-2");
    $bNamed->AddParam("LicenseeFirstName", $params["contactdetails"]["Registrant"]["First Name"]);
    $bNamed->AddParam("LicenseeLastName", $params["contactdetails"]["Registrant"]["Last Name"]);
    $bNamed->AddParam("LicenseeStreet", $params["contactdetails"]["Registrant"]["Address1"]);
    $bNamed->AddParam("LicenseePostCode", $params["contactdetails"]["Registrant"]["Postcode"]);
    $bNamed->AddParam("LicenseeResidence", $params["contactdetails"]["Registrant"]["City"]);
    $bNamed->AddParam("LicenseeCountry", $params["contactdetails"]["Registrant"]["Country"]);
    $bNamed->AddParam("LicenseePhone", $params["contactdetails"]["Registrant"]["Phone"]);
    $bNamed->AddParam("LicenseeEMail", $params["contactdetails"]["Registrant"]["E-mail"]);
    $bNamed->AddParam("LicenseeCompany", $params["contactdetails"]["Registrant"]["Company"]);

    $bNamed->AddParam("OnSiteNr", "-2");
    $bNamed->AddParam("OnSiteFirstName", $params["contactdetails"]["Tech"]["First Name"]);
    $bNamed->AddParam("OnSiteLastName", $params["contactdetails"]["Tech"]["Last Name"]);
    $bNamed->AddParam("OnSiteStreet", $params["contactdetails"]["Tech"]["Address1"]);
    $bNamed->AddParam("OnSitePostCode", $params["contactdetails"]["Tech"]["Postcode"]);
    $bNamed->AddParam("OnSiteResidence", $params["contactdetails"]["Tech"]["City"]);
    $bNamed->AddParam("OnSiteCountry", $params["contactdetails"]["Tech"]["Country"]);
    $bNamed->AddParam("OnSitePhone", $params["contactdetails"]["Tech"]["Phone"]);
    $bNamed->AddParam("OnSiteEMail", $params["contactdetails"]["Tech"]["E-mail"]);
    $bNamed->AddParam("OnSiteCompany", $params["contactdetails"]["Tech"]["Company"]);

    $bNamed->ConnUrl = str_replace(" ", "%20", $bNamed->ConnUrl);

    if (!$bNamed->SendRequest()) {
        $values["error"] = $bNamed->Error;
    }

    return $values;
}

function bnamed_GetEPPCode($params) {
    $username = $params["UID"];
    $password = $params["Key"];
    $tld = $params["tld"];
    $sld = $params["sld"];

    $bNamed = new bnamed_conn();

    $bNamed->CreateConnUrl($username, $password);
    $bNamed->AddParam("command", "lock");
    $bNamed->AddParam("sld", $sld);
    $bNamed->AddParam("tld", $tld);
    $bNamed->AddParam("lock", "0");

    if ($bNamed->SendRequest()) {
        $error = "The EPP-key has been sent via e-mail";
    } else {
        $error = $bNamed->Error;
    }

    return $error;
}

function bnamed_RegisterNameserver($params) {
    $values["error"] = "Registering your nameserver will be carried out automatically the first time you use it";
    return $values;
}

function bnamed_ModifyNameserver($params) {
    $values["error"] = "Please contact us via e-mail to update the IP-address of a nameserver";
    return $values;
}

function bnamed_DeleteNameserver($params) {
    $values["error"] = "function " . __FUNCTION__ . " not available";
    return $values;
}

function bnamed_TransferSync($params) {
    $username = $params["UID"];
    $password = $params["Key"];
    $tld = $params["tld"];
    $sld = $params["sld"];
    $values = array();

    $bNamed = new bnamed_conn();

    $bNamed->CreateConnUrl($username, $password);
    $bNamed->AddParam("command", "GetDomain");
    $bNamed->AddParam("sld", $sld);
    $bNamed->AddParam("tld", $tld);

    if ($bNamed->SendRequest()) {
        $statusCode = $bNamed->ParsedResponse->Result->StatusCode;
        if ($statusCode == "A" || $statusCode == "ANS") {
            $values['completed'] = true;
            date_default_timezone_set("Europe/Brussels");
            $values['expirydate'] = date('Y-m-d', strtotime($bNamed->ParsedResponse->Result->Expiration));
        }
    } else {
        $values["error"] = $bNamed->Error;
    }

    usleep(750000);

    return $values;
}

function bnamed_IDProtectToggle($params) {
    $username = $params["UID"];
    $password = $params["Key"];
    $tld = $params["tld"];
    $sld = $params["sld"];
    $values = array();

    $bNamed = new bnamed_conn();

    $bNamed->CreateConnUrl($username, $password);
    $bNamed->AddParam("command", "update");
    $bNamed->AddParam("sld", $sld);
    $bNamed->AddParam("tld", $tld);

    if ($params['protectenable']) {
        $bNamed->AddParam("whoisprivacy", "1");
    } else {
        $bNamed->AddParam("whoisprivacy", "0");
    }

    if (!$bNamed->SendRequest()) {
        $values["error"] = $bNamed->Error;
    }

    return $values;
}

function bnamed_Sync($params) {
    $username = $params["UID"];
    $password = $params["Key"];
    $tld = $params["tld"];
    $sld = $params["sld"];
    $values = array();

    $bNamed = new bnamed_conn();

    $bNamed->CreateConnUrl($username, $password);
    $bNamed->AddParam("command", "GetDomain");
    $bNamed->AddParam("sld", $sld);
    $bNamed->AddParam("tld", $tld);

    if ($bNamed->SendRequest()) {
        if ($bNamed->ParsedResponse->ErrorCode == 0) {
            $statusCode = $bNamed->ParsedResponse->Result->StatusCode;
            if ($statusCode == "A" || $statusCode == "ANS") {
                if ($params['idprotection'] != $bNamed->ParsedResponse->Result->WhoisPrivacy && !(strlen($bNamed->ParsedResponse->Result->WhoisPrivacy) == 0 && $params['idprotection'] == false)) {
                    $params['protectenable'] = $params['idprotection'];
                    $values = bnamed_IDProtectToggle($params);
                }

                date_default_timezone_set("Europe/Brussels");
                $values['expirydate'] = date('Y-m-d', strtotime($bNamed->ParsedResponse->Result->Expiration));
            } else {
                $values['error'] = "Name not yet active, sync only works for active domain names";
            }
        } else {
            $values['error'] = $bNamed->ParsedResponse->ErrorText;
        }
    } else {
        $values["error"] = $bNamed->Error;
    }
    usleep(750000);

    return $values;
}

function bnamed_GetDNS($params) {
    $username = $params["UID"];
    $password = $params["Key"];
    $tld = $params['tld'];
    $sld = $params['sld'];
    $values = array();

    $bNamed = new bnamed_conn();

    $bNamed->CreateConnUrl($username, $password);
    $bNamed->AddParam("command", "GetDomain");
    $bNamed->AddParam("sld", $sld);
    $bNamed->AddParam("tld", $tld);

    if ($bNamed->SendRequest()) {
        $hosts = $bNamed->ParsedResponse->Result->Host;
        if (!isset($hosts[0])) {
            $hosts = array($hosts);
        }

        foreach ($hosts->Host as $host) {
            $hostType = $host->Type;
            $hostValue = $host->Value;
            if ($hostType == "MailServerIP") {
                $hostType = "MXE";
            } elseif ($hostType == "CloackedForwarding") {
                $hostType = "FRAME";
            } elseif ($hostType == "SimpleForwarding") {
                $hostType = "URL";
            } elseif ($hostType == "EmailServices") {
                $hostType = "MX";
                $hostValue = "EmailServices";
            }

            $thisHost = array(
                'hostname' => (string) $host->attributes()->Value,
                'type' => (string) $hostType,
                'address' => (string) str_replace("\"", "", $hostValue),
                'priority' => (string) $host->Prio
            );

            $values[] = $thisHost;
        }
    } else {
        $values['error'] = $bNamed->Error;
    }

    return $values;
}

function bnamed_SaveDNS($params) {
    $username = $params["UID"];
    $password = $params["Key"];
    $tld = $params['tld'];
    $sld = $params['sld'];
    $bNamed = new bnamed_conn();

    $bNamed->CreateConnUrl($username, $password);
    $bNamed->AddParam("command", "Update");
    $bNamed->AddParam("sld", $sld);
    $bNamed->AddParam("tld", $tld);
    $bNamed->AddParam("DNSType", "B");

    $i = 0;
    foreach ($params['dnsrecords'] as $dnsrecord) {
        if (!empty($dnsrecord['hostname']) && !empty($dnsrecord['type']) && !empty($dnsrecord['address'])) {
            $hostType = $dnsrecord['type'];

            if ($hostType == "MXE") {
                $hostType = "MailServerIP";
            }

            if ($dnsrecord['type'] == "MX" && $dnsrecord['address'] == "EmailServices") {
                $hostType = "keep";
            } elseif ($hostType == "URL") {
                $hostType = "SimpleForwarding";
            } elseif ($hostType == "FRAME") {
                $hostType = "CloackedForwarding";
            }
            $bNamed->AddParam("Host" . $i . "Value", $dnsrecord['address']);
            $bNamed->AddParam("Host" . $i, $dnsrecord['hostname']);
            $bNamed->AddParam("Host" . $i . "Type", $hostType);

            if ($hostType == "MX") {
                $bNamed->AddParam("Host" . $i . "prio", $dnsrecord['priority']);
            }

            $i++;
        }
    }

    if ($i > 0) {
        if (!$bNamed->SendRequest()) {
            $values["error"] = $bNamed->Error;
        }
    }

    return $values;
}

function bnamed_SaveEmailForwarding($params) {
    $username = $params["UID"];
    $password = $params["Key"];
    $tld = $params['tld'];
    $sld = $params['sld'];
    $bNamed = new bnamed_conn();

    $bNamed->CreateConnUrl($username, $password);
    $bNamed->AddParam("command", "setEmail");
    $bNamed->AddParam("sld", $sld);
    $bNamed->AddParam("tld", $tld);
    $bNamed->AddParam("hostsetemail", "@");

    $i = 0;
    foreach ($params['prefix'] as $k => $v) {
        $from = $params['prefix'][$k];
        $to = $params['forwardto'][$k];
        if (trim($to) == '') {
            if (!is_array($params['forwardto'][$k])) {
                continue;
            }
            $from = $params['prefix'][$k][0];
            $to = $params['forwardto'][$k][0];
        }

        $bNamed->AddParam("mail" . $i . "name", $from);
        $bNamed->AddParam("mail" . $i . "forward", $to);

        $i++;
    }

    if (!$bNamed->SendRequest()) {
        $values["error"] = $bNamed->Error;
    }

    return $values;
}

function bnamed_GetEmailForwarding($params) {
    $username = $params["UID"];
    $password = $params["Key"];
    $tld = $params['tld'];
    $sld = $params['sld'];
    $values = array();

    $bNamed = new bnamed_conn();

    $bNamed->CreateConnUrl($username, $password);
    $bNamed->AddParam("command", "GetDomain");
    $bNamed->AddParam("sld", $sld);
    $bNamed->AddParam("tld", $tld);

    if ($bNamed->SendRequest()) {
        $hosts = $bNamed->ParsedResponse->Result->Host;
        if (!isset($hosts[0])) {
            $hosts = array($hosts);
        }

        $i = 0;
        foreach ($hosts->Host as $host) {
            if ($host->attributes()->Value == "@" && !empty($host->Value->Mail)) {
                foreach ($host->Value->Mail as $Mail) {
                    $values [$i] ["prefix"] = $Mail->Name;
                    $values [$i] ["forwardto"] = $Mail->Forward;

                    $i++;
                }
            }
        }
    } else {
        $values['error'] = $bNamed->Error;
    }

    return $values;
}

function bnamed_GetTldPricing(array $params) {
    // Perform API call to retrieve extension information
    // A connection error should return a simple array with error key and message
    // return ['error' => 'This error occurred',];	
    $results = new ResultsList;

    $username = $params["UID"];
    $password = $params["Key"];
    $bNamed = new bnamed_conn();

    $bNamed->CreateConnUrl($username, $password);
    $bNamed->AddParam("command", "TLDAll");

    if ($bNamed->SendRequest()) {
        $TLDs = (array) $bNamed->ParsedResponse->Result->TLDs;

        foreach ($TLDs as &$TLD) {
            //WHMCS has bug where it uses the first year price for all years of a multi-year registration
            //so we limit the maximum period to 1 year if the first year price is promotional. 
            if($bNamed->ParseFloat($TLD->Prices->PriceNew) < $bNamed->ParseFloat($TLD->Prices->PriceExtend)) {
                $periodes = Array(1);
            } else {
                $periodes = explode(",", $TLD->Registration_Period);
            }
            
            $eppRequired = false;
            $specialFieldsTransferTradeCompany = (array) $TLD->SpecialFields->TransferTrade->Company;

            if (count($specialFieldsTransferTradeCompany) > 0) {
                foreach ($specialFieldsTransferTradeCompany as &$specialfieldTransferTrade) {
                    if ($specialfieldTransferTrade[0]["name"] == "eppKey") {
                        $eppRequired = true;
                        break;
                    }
                }
            }

            // All the set methods can be chained and utilised together.
            $item = (new ImportItem)
                    ->setExtension((string) $TLD->TLD)
                    ->setRegisterPrice($bNamed->ParseFloat($TLD->Prices->PriceNew))
                    ->setRenewPrice($bNamed->ParseFloat($TLD->Prices->PriceExtend))
                    ->setTransferPrice($bNamed->ParseFloat($TLD->Prices->PriceTransfer))
                    ->setGraceFeeDays((int) $TLD->Reactivation->Period_1->Term)
                    ->setGraceFeePrice($bNamed->ParseFloat($TLD->Reactivation->Period_1->Surcharge))
                    ->setRedemptionFeeDays((int) $TLD->Reactivation->Period_2->Term)
                    ->setRedemptionFeePrice($bNamed->ParseFloat($TLD->Reactivation->Period_2->Surcharge))
                    ->setCurrency((string) $TLD->Prices->Currency)
                    ->setEppRequired((boolean) $eppRequired)
                    ->setYears($periodes);

            $results[] = $item;
        }

        return $results;
    } else {
        return ['error' => $bNamed->Error,];
    }
}

?>
